@@ -2,7 +2,7 @@
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2012-2015 Google, Inc.
 # Copyright (c) 2013 moxian <aleftmail@inbox.ru>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2019 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 frost-nzcr4 <frost.nzcr4@jagmort.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
@@ -16,20 +16,27 @@
 # Copyright (c) 2016 Petr Pulc <petrpulc@gmail.com>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2017, 2019 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017-2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Krzysztof Czapla <k.czapla68@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 James M. Allen <james.m.allen@gmail.com>
 # Copyright (c) 2017 vinnyrose <vinnyrose@users.noreply.github.com>
+# Copyright (c) 2018-2020 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2018 Pierre Sassoulas <pierre.sassoulas@wisebim.fr>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Michael Hudson-Doyle <michael.hudson@canonical.com>
+# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Marcus Näslund <naslundx@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Fureigh <rhys.fureigh@gsa.gov>
-# Copyright (c) 2018 Pierre Sassoulas <pierre.sassoulas@wisebim.fr>
 # Copyright (c) 2018 Andreas Freimuth <andreas.freimuth@united-bits.de>
 # Copyright (c) 2018 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -44,16 +51,22 @@ Some parts of the process_token method is based from The Tab Nanny std module.
 """
 
 import keyword
-import sys
 import tokenize
 from functools import reduce  # pylint: disable=redefined-builtin
+from typing import List
 
 from astroid import nodes
 
-from pylint.interfaces import ITokenChecker, IAstroidChecker, IRawChecker
 from pylint.checkers import BaseTokenChecker
-from pylint.checkers.utils import check_messages
-from pylint.utils import WarningScope, OPTION_RGX
+from pylint.checkers.utils import (
+    check_messages,
+    is_overload_stub,
+    is_protocol_class,
+    node_frame_class,
+)
+from pylint.constants import WarningScope
+from pylint.interfaces import IAstroidChecker, IRawChecker, ITokenChecker
+from pylint.utils.pragma_parser import OPTION_PO, PragmaParserError, parse_pragma
 
 _ASYNC_TOKEN = "async"
 _CONTINUATION_BLOCK_OPENERS = [
@@ -81,8 +94,6 @@ _KEYWORD_TOKENS = [
     "yield",
     "with",
 ]
-if sys.version_info < (3, 0):
-    _KEYWORD_TOKENS.append("print")
 
 _SPACED_OPERATORS = [
     "==",
@@ -273,8 +284,8 @@ def _get_indent_length(line):
 def _get_indent_hint_line(bar_positions, bad_position):
     """Return a line with |s for each of the positions in the given lists."""
     if not bar_positions:
-        return ("", "")
-    # TODO tabs should not be replaced by some random (8) number of spaces
+        return "", ""
+
     bar_positions = [_get_indent_length(indent) for indent in bar_positions]
     bad_position = _get_indent_length(bad_position)
     delta_message = ""
@@ -294,7 +305,7 @@ def _get_indent_hint_line(bar_positions, bad_position):
     line = [" "] * (markers[-1][0] + 1)
     for position, marker in markers:
         line[position] = marker
-    return ("".join(line), delta_message)
+    return "".join(line), delta_message
 
 
 class _ContinuedIndent:
@@ -724,10 +735,8 @@ class FormatChecker(BaseTokenChecker):
             self._lines[line_num] = line.split("\n")[0]
         self.check_lines(line, line_num)
 
-    def process_module(self, module):
+    def process_module(self, _module):
         self._keywords_with_parens = set()
-        if "print_function" in module.future_imports:
-            self._keywords_with_parens.add("print")
 
     def _check_keyword_parentheses(self, tokens, start):
         """Check that there are not unnecessary parens after a keyword.
@@ -1170,22 +1179,20 @@ class FormatChecker(BaseTokenChecker):
         if not node.is_statement:
             return
         if not node.root().pure_python:
-            return  # XXX block visit of child nodes
+            return
         prev_sibl = node.previous_sibling()
         if prev_sibl is not None:
             prev_line = prev_sibl.fromlineno
+        # The line on which a finally: occurs in a try/finally
+        # is not directly represented in the AST. We infer it
+        # by taking the last line of the body and adding 1, which
+        # should be the line of finally:
+        elif (
+            isinstance(node.parent, nodes.TryFinally) and node in node.parent.finalbody
+        ):
+            prev_line = node.parent.body[0].tolineno + 1
         else:
-            # The line on which a finally: occurs in a try/finally
-            # is not directly represented in the AST. We infer it
-            # by taking the last line of the body and adding 1, which
-            # should be the line of finally:
-            if (
-                isinstance(node.parent, nodes.TryFinally)
-                and node in node.parent.finalbody
-            ):
-                prev_line = node.parent.body[0].tolineno + 1
-            else:
-                prev_line = node.parent.statement().fromlineno
+            prev_line = node.parent.statement().fromlineno
         line = node.fromlineno
         assert line, node
         if prev_line == line and self._visited_lines.get(line) != 2:
@@ -1230,44 +1237,77 @@ class FormatChecker(BaseTokenChecker):
             and self.config.single_line_class_stmt
         ):
             return
+
+        # Function overloads that use ``Ellipsis`` are exempted.
+        if isinstance(node, nodes.Expr) and (
+            isinstance(node.value, nodes.Ellipsis)
+            or (isinstance(node.value, nodes.Const) and node.value.value is Ellipsis)
+        ):
+            frame = node.frame()
+            if is_overload_stub(frame) or is_protocol_class(node_frame_class(frame)):
+                return
+
         self.add_message("multiple-statements", node=node)
         self._visited_lines[line] = 2
 
-    def check_lines(self, lines, i):
-        """check lines have less than a maximum number of characters
+    def check_line_ending(self, line: str, i: int) -> None:
+        """
+        Check that the final newline is not missing and that there is no trailing whitespace.
+        """
+        if not line.endswith("\n"):
+            self.add_message("missing-final-newline", line=i)
+        else:
+            # exclude \f (formfeed) from the rstrip
+            stripped_line = line.rstrip("\t\n\r\v ")
+            if not stripped_line and _EMPTY_LINE in self.config.no_space_check:
+                # allow empty lines
+                pass
+            elif line[len(stripped_line) :] not in ("\n", "\r\n"):
+                self.add_message(
+                    "trailing-whitespace", line=i, col_offset=len(stripped_line)
+                )
+
+    def check_line_length(self, line: str, i: int) -> None:
+        """
+        Check that the line length is less than the authorized value
         """
         max_chars = self.config.max_line_length
         ignore_long_line = self.config.ignore_long_lines
+        line = line.rstrip()
+        if len(line) > max_chars and not ignore_long_line.search(line):
+            self.add_message("line-too-long", line=i, args=(len(line), max_chars))
 
-        def check_line(line, i):
-            if not line.endswith("\n"):
-                self.add_message("missing-final-newline", line=i)
-            else:
-                # exclude \f (formfeed) from the rstrip
-                stripped_line = line.rstrip("\t\n\r\v ")
-                if not stripped_line and _EMPTY_LINE in self.config.no_space_check:
-                    # allow empty lines
-                    pass
-                elif line[len(stripped_line) :] not in ("\n", "\r\n"):
-                    self.add_message(
-                        "trailing-whitespace", line=i, col_offset=len(stripped_line)
-                    )
-                # Don't count excess whitespace in the line length.
-                line = stripped_line
-            mobj = OPTION_RGX.search(line)
-            if mobj and "=" in line:
-                front_of_equal, _, back_of_equal = mobj.group(1).partition("=")
-                if front_of_equal.strip() == "disable":
-                    if "line-too-long" in {
-                        _msg_id.strip() for _msg_id in back_of_equal.split(",")
-                    }:
-                        return None
-                    line = line.rsplit("#", 1)[0].rstrip()
+    @staticmethod
+    def remove_pylint_option_from_lines(options_pattern_obj) -> str:
+        """
+        Remove the `# pylint ...` pattern from lines
+        """
+        lines = options_pattern_obj.string
+        purged_lines = (
+            lines[: options_pattern_obj.start(1)].rstrip()
+            + lines[options_pattern_obj.end(1) :]
+        )
+        return purged_lines
 
-            if len(line) > max_chars and not ignore_long_line.search(line):
-                self.add_message("line-too-long", line=i, args=(len(line), max_chars))
-            return i + 1
+    @staticmethod
+    def is_line_length_check_activated(pylint_pattern_match_object) -> bool:
+        """
+        Return true if the line length check is activated
+        """
+        try:
+            for pragma in parse_pragma(pylint_pattern_match_object.group(2)):
+                if pragma.action == "disable" and "line-too-long" in pragma.messages:
+                    return False
+        except PragmaParserError:
+            # Printing useful information dealing with this error is done in the lint package
+            pass
+        return True
 
+    @staticmethod
+    def specific_splitlines(lines: str) -> List[str]:
+        """
+        Split lines according to universal newlines except those in a specific sets
+        """
         unsplit_ends = {
             "\v",
             "\x0b",
@@ -1280,23 +1320,38 @@ class FormatChecker(BaseTokenChecker):
             "\u2028",
             "\u2029",
         }
-        unsplit = []
-        for line in lines.splitlines(True):
-            if line[-1] in unsplit_ends:
-                unsplit.append(line)
-                continue
+        res = []
+        buffer = ""
+        for atomic_line in lines.splitlines(True):
+            if atomic_line[-1] not in unsplit_ends:
+                res.append(buffer + atomic_line)
+                buffer = ""
+            else:
+                buffer += atomic_line
+        return res
 
-            if unsplit:
-                unsplit.append(line)
-                line = "".join(unsplit)
-                unsplit = []
+    def check_lines(self, lines: str, lineno: int) -> None:
+        """
+        Check lines have :
+            - a final newline
+            - no trailing whitespace
+            - less than a maximum number of characters
+        """
+        #  By default, check the line length
+        check_l_length = True
 
-            i = check_line(line, i)
-            if i is None:
-                break
+        # Line length check may be deactivated through `pylint: disable` comment
+        mobj = OPTION_PO.search(lines)
+        if mobj:
+            check_l_length = self.is_line_length_check_activated(mobj)
+            # The 'pylint: disable whatever' should not be taken into account for line length count
+            lines = self.remove_pylint_option_from_lines(mobj)
 
-        if unsplit:
-            check_line("".join(unsplit), i)
+        for line in self.specific_splitlines(lines):
+            if check_l_length:
+                self.check_line_length(line, lineno)
+            self.check_line_ending(line, lineno)
+            lineno += 1
 
     def check_indent_level(self, string, expected, line_num):
         """return the indent level of the string
@@ -1335,9 +1390,3 @@ class FormatChecker(BaseTokenChecker):
 def register(linter):
     """required method to auto register this checker """
     linter.register_checker(FormatChecker(linter))
-
-
-def funkier():
-    for val in range(3):
-        pass
-    print(val)
