@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2019 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014-2015 Brett Cannon <brett@python.org>
 # Copyright (c) 2015 Simu Toni <simutoni@gmail.com>
 # Copyright (c) 2015 Pavel Roskin <proski@gnu.org>
@@ -11,19 +11,30 @@
 # Copyright (c) 2016 Roy Williams <rwilliams@lyft.com>
 # Copyright (c) 2016 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2016 Erik <erik.eriksson@yahoo.com>
-# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2017-2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
 # Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 ahirnish <ahirnish@gmail.com>
+# Copyright (c) 2018-2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 sbagan <pnlbagan@gmail.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Aivar Annamaa <aivarannamaa@users.noreply.github.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 # Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
 # Copyright (c) 2018 gaurikholkar <f2013002@goa.bits-pilani.ac.in>
+# Copyright (c) 2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Hugues Bruant <hugues.bruant@affirm.com>
+# Copyright (c) 2019 Gabriel R Sezefredo <gabriel@sezefredo.com.br>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 bluesheeptoken <louis.fruleux1@gmail.com>
+# Copyright (c) 2020 Athos Ribeiro <athoscr@fedoraproject.org>
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """Check Python 2 code for Python 2/3 source-compatible issues."""
+import itertools
 import re
 import tokenize
 from collections import namedtuple
@@ -58,6 +69,10 @@ def _inferred_value_is_dict(value):
     return isinstance(value, astroid.Instance) and "dict" in value.basenames
 
 
+def _infer_if_relevant_attr(node, whitelist):
+    return node.expr.infer() if node.attrname in whitelist else []
+
+
 def _is_builtin(node):
     return getattr(node, "name", None) in ("__builtin__", "builtins")
 
@@ -134,7 +149,7 @@ def _in_iterating_context(node):
     elif (
         isinstance(parent, astroid.Compare)
         and len(parent.ops) == 1
-        and parent.ops[0][0] == "in"
+        and parent.ops[0][0] in ["in", "not in"]
     ):
         return True
     # Also if it's an `yield from`, that's fair
@@ -873,6 +888,19 @@ class Python3Checker(checkers.BaseChecker):
         "deprecated-sys-function": {"sys": frozenset({"exc_clear"})},
     }
 
+    _deprecated_attrs = frozenset(
+        itertools.chain.from_iterable(
+            attr
+            for module_map in _bad_python3_module_map.values()
+            if isinstance(module_map, dict)
+            for attr in module_map.values()
+        )
+    )
+
+    _relevant_call_attrs = (
+        DICT_METHODS | _deprecated_attrs | {"encode", "decode", "translate"}
+    )
+
     _python_2_tests = frozenset(
         [
             astroid.extract_node(x).repr_tree()
@@ -891,14 +919,14 @@ class Python3Checker(checkers.BaseChecker):
         self._future_absolute_import = False
         self._modules_warned_about = set()
         self._branch_stack = []
-        super(Python3Checker, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     # pylint: disable=keyword-arg-before-vararg, arguments-differ
     def add_message(self, msg_id, always_warn=False, *args, **kwargs):
         if always_warn or not (
             self._branch_stack and self._branch_stack[-1].is_py2_only
         ):
-            super(Python3Checker, self).add_message(msg_id, *args, **kwargs)
+            super().add_message(msg_id, *args, **kwargs)
 
     def _is_py2_test(self, node):
         if isinstance(node.test, astroid.Attribute) and isinstance(
@@ -1132,8 +1160,11 @@ class Python3Checker(checkers.BaseChecker):
 
         if isinstance(node.func, astroid.Attribute):
             inferred_types = set()
+
             try:
-                for inferred_receiver in node.func.expr.infer():
+                for inferred_receiver in _infer_if_relevant_attr(
+                    node.func, self._relevant_call_attrs
+                ):
                     if inferred_receiver is astroid.Uninferable:
                         continue
                     inferred_types.add(inferred_receiver)
@@ -1191,11 +1222,10 @@ class Python3Checker(checkers.BaseChecker):
                 return
             if node.func.attrname == "next":
                 self.add_message("next-method-called", node=node)
-            else:
-                if node.func.attrname in ("iterkeys", "itervalues", "iteritems"):
-                    self.add_message("dict-iter-method", node=node)
-                elif node.func.attrname in ("viewkeys", "viewvalues", "viewitems"):
-                    self.add_message("dict-view-method", node=node)
+            elif node.func.attrname in ("iterkeys", "itervalues", "iteritems"):
+                self.add_message("dict-iter-method", node=node)
+            elif node.func.attrname in ("viewkeys", "viewvalues", "viewitems"):
+                self.add_message("dict-view-method", node=node)
         elif isinstance(node.func, astroid.Name):
             found_node = node.func.lookup(node.func.name)[0]
             if _is_builtin(found_node):
@@ -1244,7 +1274,9 @@ class Python3Checker(checkers.BaseChecker):
 
         exception_message = "message"
         try:
-            for inferred in node.expr.infer():
+            for inferred in _infer_if_relevant_attr(
+                node, self._deprecated_attrs | {exception_message}
+            ):
                 if isinstance(inferred, astroid.Instance) and utils.inherit_from_std_ex(
                     inferred
                 ):
@@ -1265,16 +1297,11 @@ class Python3Checker(checkers.BaseChecker):
     def visit_excepthandler(self, node):
         """Visit an except handler block and check for exception unpacking."""
 
-        def _is_used_in_except_block(node):
-            scope = node.scope()
+        def _is_used_in_except_block(node, block):
             current = node
-            while (
-                current
-                and current != scope
-                and not isinstance(current, astroid.ExceptHandler)
-            ):
+            while current and current is not block:
                 current = current.parent
-            return isinstance(current, astroid.ExceptHandler) and current.type != node
+            return current is not None
 
         if isinstance(node.name, (astroid.Tuple, astroid.List)):
             self.add_message("unpacking-in-except", node=node)
@@ -1292,7 +1319,7 @@ class Python3Checker(checkers.BaseChecker):
             for scope_name in scope_names
             if scope_name.name == node.name.name
             and scope_name.lineno > node.lineno
-            and not _is_used_in_except_block(scope_name)
+            and not _is_used_in_except_block(scope_name, node)
         ]
         reassignments_for_same_name = {
             assign_name.lineno
