@@ -1,15 +1,18 @@
-# -*- coding: utf-8 -*-
-# Copyright (c) 2016-2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2019 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Yuri Bochkarev <baltazar.bz@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Mitar <mitar.github@tnode.com>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Mitchell T.H. Young <mitchelly@gmail.com>
 # Copyright (c) 2018 Adrian Chirieac <chirieacam@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Danny Hermes <daniel.j.hermes@gmail.com>
+# Copyright (c) 2019 Zeb Nicholls <zebedee.nicholls@climate-energy-college.org>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -17,6 +20,7 @@
 """Utility methods for docstring checking."""
 
 import re
+from typing import List
 
 import astroid
 
@@ -105,6 +109,11 @@ def _get_raise_target(node):
     return None
 
 
+def _split_multiple_exc_types(target: str) -> List[str]:
+    delimiters = r"(\s*,(?:\s*or\s)?\s*|\s+or\s+)"
+    return re.split(delimiters, target)
+
+
 def possible_exc_types(node):
     """
     Gets all of the possible raised exception types for the given raise node.
@@ -239,6 +248,13 @@ class SphinxDocstring(Docstring):
         type=re_type
     )
 
+    re_multiple_simple_type = r"""
+        (?:{container_type}|{type})
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}))*
+    """.format(
+        type=re_type, container_type=re_simple_container_type
+    )
+
     re_xref = r"""
         (?::\w+:)?                    # optional tag
         `{}`                         # what to reference
@@ -275,7 +291,7 @@ class SphinxDocstring(Docstring):
         \s*                     # whitespace
         :                       # final colon
         """.format(
-        type=re_type
+        type=re_multiple_simple_type
     )
     re_type_in_docstring = re.compile(re_type_raw, re.X | re.S)
 
@@ -284,7 +300,7 @@ class SphinxDocstring(Docstring):
         \s+                     # whitespace
         {type}                  # type declaration
         """.format(
-        type=re_type
+        type=re_multiple_simple_type
     )
     re_property_type_in_docstring = re.compile(re_property_type_raw, re.X | re.S)
 
@@ -299,7 +315,7 @@ class SphinxDocstring(Docstring):
         \s*                     # whitespace
         :                       # final colon
         """.format(
-        type=re_type
+        type=re_multiple_simple_type
     )
     re_raise_in_docstring = re.compile(re_raise_raw, re.X | re.S)
 
@@ -323,7 +339,7 @@ class SphinxDocstring(Docstring):
 
         for match in re.finditer(self.re_raise_in_docstring, self.doc):
             raise_type = match.group(1)
-            types.add(raise_type)
+            types.update(_split_multiple_exc_types(raise_type))
 
         return types
 
@@ -441,7 +457,7 @@ class GoogleDocstring(Docstring):
 
     re_multiple_type = r"""
         (?:{container_type}|{type}|{xref})
-        (?:\s+(?:of|or)\s+(?:{container_type}|{type}|{xref}))*
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}|{xref}))*
     """.format(
         type=re_type, xref=re_xref, container_type=re_container_type
     )
@@ -484,7 +500,7 @@ class GoogleDocstring(Docstring):
         \s*  ({type}) \s* :              # identifier
         \s*  (.*)                        # beginning of optional description
     """.format(
-            type=re_type
+            type=re_multiple_type
         ),
         re.X | re.S | re.M,
     )
@@ -629,7 +645,7 @@ class GoogleDocstring(Docstring):
             exc_type = match.group(1)
             exc_desc = match.group(2)
             if exc_desc:
-                types.add(exc_type)
+                types.update(_split_multiple_exc_types(exc_type))
 
         return types
 
