@@ -1,7 +1,10 @@
-# Copyright (c) 2015-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2020 Benjamin Graham <benwilliamgraham@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -10,12 +13,12 @@
 
 (dot generation adapted from pypy/translator/tool/make_dot.py)
 """
-
-import os.path as osp
+import codecs
 import os
+import subprocess
 import sys
 import tempfile
-import codecs
+from os import path as osp
 
 
 def target_info_from_filename(filename):
@@ -70,32 +73,23 @@ class DotBackend:
 
     source = property(get_source)
 
-    def generate(self, outputfile=None, dotfile=None, mapfile=None):
+    def generate(self, outputfile=None, mapfile=None):
         """Generates a graph file.
 
         :param str outputfile: filename and path [defaults to graphname.png]
-        :param str dotfile: filename and path [defaults to graphname.dot]
         :param str mapfile: filename and path
 
         :rtype: str
         :return: a path to the generated file
         """
-        import subprocess  # introduced in py 2.4
-
         name = self.graphname
-        if not dotfile:
-            # if 'outputfile' is a dot file use it as 'dotfile'
-            if outputfile and outputfile.endswith(".dot"):
-                dotfile = outputfile
-            else:
-                dotfile = "%s.dot" % name
         if outputfile is not None:
-            storedir, _, target = target_info_from_filename(outputfile)
+            _, _, target = target_info_from_filename(outputfile)
             if target != "dot":
                 pdot, dot_sourcepath = tempfile.mkstemp(".dot", name)
                 os.close(pdot)
             else:
-                dot_sourcepath = osp.join(storedir, dotfile)
+                dot_sourcepath = outputfile
         else:
             target = "png"
             pdot, dot_sourcepath = tempfile.mkstemp(".dot", name)
